from swmmio.graphics.constants import *

FLOOD_IMPACT_CATEGORIES = {
    'increased_flooding':{
        'fill':red,
    },
    'new_flooding':{
        'fill':purple,
    },
    'decreased_flooding':{
        'fill':lightblue,
    },
    'eliminated_flooding':{
        'fill':lightgreen,
    },

}
